@@ -83,6 +83,8 @@ pid_t extra_process = 0;	/* An extra process.  Used for quickie
 				   processes, so that they also get whacked */
 int extra_status = 0;		/* Last status from the last extra process */
 pid_t gdm_main_pid = 0;		/* PID of the main daemon */
+
+gboolean print_version = FALSE; /* print version number and quit */
 gboolean preserve_ld_vars = FALSE; /* Preserve the ld environment variables */
 gboolean no_daemon = FALSE;	/* Do not daemonize */
 gboolean no_console = FALSE;	/* There are no local servers, this means,
@@ -117,7 +119,6 @@ gchar *GdmGroup = NULL;
 gchar *GdmGtkRC = NULL;
 gchar *GdmSessDir = NULL;
 gchar *GdmXsession = NULL;
-gchar *GdmLocaleFile = NULL;
 gchar *GdmAutomaticLogin = NULL;
 gboolean GdmAutomaticLoginEnable = FALSE;
 gboolean GdmAlwaysRestartServer = FALSE;
@@ -293,11 +294,6 @@ gdm_config_parse (void)
     displays = NULL;
     high_display_num = 0;
 
-#if defined(_SCO_DS) || defined(__UNIXWARE__) || defined(__USLC__)
-    gdm_error ("SCO OS detected!!!  I will sleep for 5 seconds in protest.  This message can be disabled by SCO stopping to sue people.");
-    sleep (5);
-#endif
-
     IGNORE_EINTR (r = stat (GDM_CONFIG_FILE, &statbuf));
     if (r < 0) {
 	    gdm_error (_("%s: No configuration file: %s. Using defaults."),
@@ -349,11 +345,10 @@ gdm_config_parse (void)
 			"gdm_config_parse",
 			EXPANDED_SYSCONFDIR);
 	    g_free (GdmXsession);
-	    GdmXsession = g_strconcat (EXPANDED_SYSCONFDIR, "/gdm/Xsession",
-				       NULL);
+	    GdmXsession = g_build_filename (EXPANDED_SYSCONFDIR,
+					    "gdm", "Xsession", NULL);
     }
     GdmSuspend = ve_config_get_string (cfg, GDM_KEY_SUSPEND);
-    GdmLocaleFile = ve_config_get_string (cfg, GDM_KEY_LOCFILE);
     GdmUser = ve_config_get_string (cfg, GDM_KEY_USER);
     GdmUserAuthDir = ve_config_get_string (cfg, GDM_KEY_UAUTHDIR);
     GdmUserAuthFile = ve_config_get_string (cfg, GDM_KEY_UAUTHFILE);
@@ -612,7 +607,7 @@ gdm_config_parse (void)
     /* Lookup user and groupid for the gdm user */
     pwent = getpwnam (GdmUser);
 
-    if G_UNLIKELY (pwent == NULL) {
+    if G_UNLIKELY (pwent == NULL && 0 /* FIXME: this is evil! */) {
 	    gdm_error (_("%s: Can't find the gdm user (%s). Trying 'nobody'!"), "gdm_config_parse", GdmUser);
 	    g_free (GdmUser);
 	    GdmUser = g_strdup ("nobody");
@@ -648,7 +643,7 @@ gdm_config_parse (void)
 
     grent = getgrnam (GdmGroup);
 
-    if G_UNLIKELY (grent == NULL) {
+    if G_UNLIKELY (grent == NULL && 0 /* FIXME: this is evil! */) {
 	    gdm_error (_("%s: Can't find the gdm group (%s). Trying 'nobody'!"), "gdm_config_parse", GdmUser);
 	    g_free (GdmGroup);
 	    GdmGroup = g_strdup ("nobody");
@@ -832,7 +827,7 @@ gdm_daemonify (void)
 	    errno = 0;
 	    fprintf (pf, "%d\n", (int)pid);
 	    fclose (pf);
-	    if (errno != 0) {
+	    if G_UNLIKELY (errno != 0) {
 		    /* FIXME: how to handle this? */
 		    gdm_fdprintf (2, "Cannot write PID file %s, possibly out of diskspace.  Error: %s\n",
 				  GdmPidFile, strerror (errno));
@@ -840,7 +835,7 @@ gdm_daemonify (void)
 			       GdmPidFile, strerror (errno));
 
 	    }
-	} else if (errno != 0) {
+	} else if G_UNLIKELY (errno != 0) {
 	    /* FIXME: how to handle this? */
 	    gdm_fdprintf (2, "Cannot write PID file %s, possibly out of diskspace.  Error: %s\n",
 			  GdmPidFile, strerror (errno));
@@ -987,7 +982,7 @@ gdm_final_cleanup (void)
 	if (fifoconn != NULL) {
 		char *path;
 		gdm_connection_close (fifoconn);
-		path = g_strconcat (GdmServAuthDir, "/.gdmfifo", NULL);
+		path = g_build_filename (GdmServAuthDir, ".gdmfifo", NULL);
 		IGNORE_EINTR (unlink (path));
 		g_free (path);
 		fifoconn = NULL;
@@ -1055,7 +1050,7 @@ deal_with_x_crashes (GdmDisplay *d)
 
 	    if (pid == 0) {
 		    char *argv[2];
-		    char *xlog = g_strconcat (GdmLogDir, "/", d->name, ".log", NULL);
+		    char *xlog = gdm_make_filename (GdmLogDir, d->name, ".log");
 
 		    gdm_unset_signals ();
 
@@ -1656,7 +1651,7 @@ create_connections (void)
 	int p[2];
 	gchar *path;
 
-	path = g_strconcat (GdmServAuthDir, "/.gdmfifo", NULL);
+	path = g_build_filename (GdmServAuthDir, ".gdmfifo", NULL);
 	fifoconn = gdm_connection_open_fifo (path, 0660);
 	g_free (path);
 
@@ -1713,6 +1708,8 @@ struct poptOption options [] = {
 	  &no_console, 0, N_("No console (local) servers to be run"), NULL },
 	{ "preserve-ld-vars", '\0', POPT_ARG_NONE,
 	  &preserve_ld_vars, 0, N_("Preserve LD_* variables"), NULL },
+	{ "version", '\0', POPT_ARG_NONE,
+	  &print_version, 0, NULL, NULL },
         POPT_AUTOHELP
 	{ NULL, 0, 0, NULL, 0}
 };
@@ -1802,7 +1799,10 @@ main (int argc, char *argv[])
     int nextopt;
     const char *charset;
 
+    /* semi init pseudorandomness */
     gdm_random_tick ();
+    srand (g_random_int ());
+    srandom (g_random_int ());
 
     gdm_main_pid = getpid ();
 
@@ -1834,10 +1834,16 @@ main (int argc, char *argv[])
 		     poptBadOption (ctx, 0),
 		     poptStrerror (nextopt),
 		     argv[0]);
-	    fflush (stdout);
+	    fflush (stderr);
 	    exit (1);
     }
 
+    if (print_version) {
+	    printf ("GDM %s\n", VERSION);
+	    fflush (stdout);
+	    exit (0);
+    }
+
     /* XDM compliant error message */
     if G_UNLIKELY (getuid () != 0) {
 	    /* make sure the pid file doesn't get wiped */
@@ -2080,7 +2086,7 @@ static void
 write_x_servers (GdmDisplay *d)
 {
 	FILE *fp;
-	char *file = g_strconcat (GdmServAuthDir, "/", d->name, ".Xservers", NULL);
+	char *file = gdm_make_filename (GdmServAuthDir, d->name, ".Xservers");
 	int i;
 	int bogusname;
 
