@@ -45,22 +45,24 @@
  * process exit codes */
 #define SERVER_TIMEOUT 2	/* Server didn't start */
 #define SERVER_DEAD 250		/* Server stopped */
-#define SERVER_STARTED 251	/* Server started but not ready for connections yet */
+#define SERVER_PENDING 251	/* Server started but not ready for connections yet */
 #define SERVER_RUNNING 252	/* Server running and ready for connections */
 #define SERVER_ABORT 253	/* Server failed badly. Suspending display. */
 
 /* DO NOTE USE 1, that's used as error if x connection fails usually */
-/* Note that there is no reasons why these were a power of two, and note
+/* Note that there is no reason why these were a power of two, and note
  * that they have to fit in 256 */
 /* These are the exit codes */
 #define DISPLAY_REMANAGE 2	/* Restart display */
 #define DISPLAY_ABORT 4		/* Houston, we have a problem */
 #define DISPLAY_REBOOT 8	/* Rebewt */
 #define DISPLAY_HALT 16		/* Halt */
-#define DISPLAY_SUSPEND 17	/* Suspend */
+#define DISPLAY_SUSPEND 17	/* Suspend (don't use, use the interrupt) */
 #define DISPLAY_CHOSEN 20	/* successful chooser session,
 				   restart display */
+#define DISPLAY_RUN_CHOOSER 30	/* Run chooser */
 #define DISPLAY_XFAILED 64	/* X failed */
+#define DISPLAY_GREETERFAILED 65 /* greeter failed (crashed) */
 #define DISPLAY_RESTARTGREETER 127 /* Restart greeter */
 #define DISPLAY_RESTARTGDM 128	/* Restart GDM */
 
@@ -77,7 +79,7 @@ enum {
 
 /* This will change if there are incompatible
  * protocol changes */
-#define GDM_GREETER_PROTOCOL_VERSION "2"
+#define GDM_GREETER_PROTOCOL_VERSION "3"
 
 #define GDM_MSG        'D'
 #define GDM_NOECHO     'U'
@@ -89,12 +91,8 @@ enum {
 #define GDM_RESET      'A'
 #define GDM_QUIT       'P'
 /* Well these aren't as nice as above, oh well */
-#define GDM_GNOMESESS  '?'
-#define GDM_SGNOMESESS '*'
 #define GDM_STARTTIMER 's'
 #define GDM_STOPTIMER  'S'
-#define GDM_LOGIN      'L' /* this is the login prompt, much like PROMPT but
-			      different */
 #define GDM_SETLOGIN   'l' /* this just sets the login to be this, just for
 			      the greeters knowledge */
 #define GDM_DISABLE    '-' /* disable the login screen */
@@ -108,10 +106,13 @@ enum {
 #define GDM_NOFOCUS    'f' /* Don't focus the login window (optional) */
 #define GDM_FOCUS      'F' /* Allow focus on the login window again (optional) */
 #define GDM_SAVEDIE    '!' /* Save wm order and die (and set busy cursor) */
+#define GDM_QUERY_CAPSLOCK 'Q' /* Is capslock on? */
 
 /* Different login interruptions */
 #define GDM_INTERRUPT_TIMED_LOGIN 'T'
 #define GDM_INTERRUPT_CONFIGURE   'C'
+#define GDM_INTERRUPT_SUSPEND     'S'
+#define GDM_INTERRUPT_SELECT_USER 'U'
 
 /* The dreaded miscellaneous category */
 #define FIELD_SIZE 256
@@ -124,26 +125,29 @@ enum {
  * to nothing */
 #define GDM_KEY_AUTOMATICLOGIN_ENABLE "daemon/AutomaticLoginEnable=true"
 #define GDM_KEY_AUTOMATICLOGIN "daemon/AutomaticLogin="
-#define GDM_KEY_LOCALNOPASSWORDUSERS "daemon/LocalNoPasswordUsers="
-#define GDM_KEY_ALWAYSRESTARTSERVER "daemon/AlwaysRestartServer=true"
+#define GDM_KEY_ALWAYSRESTARTSERVER "daemon/AlwaysRestartServer=false"
 #define GDM_KEY_GREETER "daemon/Greeter=" EXPANDED_BINDIR "/gdmlogin"
 #define GDM_KEY_REMOTEGREETER "daemon/RemoteGreeter=" EXPANDED_BINDIR "/gdmlogin"
+#define GDM_KEY_ADD_GTK_MODULES "daemon/AddGtkModules=false"
+#define GDM_KEY_GTK_MODULES_LIST "daemon/GtkModulesList="
 #define GDM_KEY_GROUP "daemon/Group=gdm"
 #define GDM_KEY_HALT "daemon/HaltCommand=/usr/bin/poweroff;/sbin/poweroff;/sbin/shutdown -h now;/usr/sbin/shutdown -h now"
 #define GDM_KEY_INITDIR "daemon/DisplayInitDir=" EXPANDED_SYSCONFDIR "/gdm/Init"
 #define GDM_KEY_KILLIC "daemon/KillInitClients=true"
-#define GDM_KEY_LOGDIR "daemon/LogDir=" EXPANDED_AUTHDIR
-#define GDM_KEY_PATH "daemon/DefaultPath=/bin:/usr/bin:/usr/bin/X11:/usr/local/bin:" EXPANDED_BINDIR
+#define GDM_KEY_LOGDIR "daemon/LogDir=" EXPANDED_LOGDIR
+#define GDM_KEY_PATH "daemon/DefaultPath=/bin:/usr/bin:" X_CONF_PATH ":" EXPANDED_BINDIR
 #define GDM_KEY_PIDFILE "daemon/PidFile=/var/run/gdm.pid"
 #define GDM_KEY_POSTSESS "daemon/PostSessionScriptDir=" EXPANDED_SYSCONFDIR "/gdm/PostSession/"
 #define GDM_KEY_PRESESS "daemon/PreSessionScriptDir=" EXPANDED_SYSCONFDIR "/gdm/PreSession/"
+#define GDM_KEY_POSTLOGIN "daemon/PostLoginScriptDir=" EXPANDED_SYSCONFDIR "/gdm/PreSession/"
 #define GDM_KEY_FAILSAFE_XSERVER "daemon/FailsafeXServer="
 #define GDM_KEY_XKEEPSCRASHING "daemon/XKeepsCrashing=" EXPANDED_SYSCONFDIR "/gdm/XKeepsCrashing"
 #define GDM_KEY_REBOOT "daemon/RebootCommand=/usr/bin/reboot;/sbin/reboot;/sbin/shutdown -r now;/usr/sbin/shutdown -r now"
-#define GDM_KEY_ROOTPATH "daemon/RootPath=/sbin:/usr/sbin:/bin:/usr/bin:/usr/bin/X11:/usr/local/bin:" EXPANDED_BINDIR
-#define GDM_KEY_GNOMEDEFAULTSESSION "daemon/GnomeDefaultSession=" EXPANDED_DATADIR "/gnome/default.session"
+#define GDM_KEY_ROOTPATH "daemon/RootPath=/sbin:/usr/sbin:/bin:/usr/bin:" X_CONF_PATH ":" EXPANDED_BINDIR
 #define GDM_KEY_SERVAUTH "daemon/ServAuthDir=" EXPANDED_AUTHDIR
-#define GDM_KEY_SESSDIR "daemon/SessionDir=" EXPANDED_SYSCONFDIR "/gdm/Sessions/"
+#define GDM_KEY_SESSDIR "daemon/SessionDesktopDir=" EXPANDED_SYSCONFDIR "/dm/Sessions/"
+#define GDM_KEY_BASEXSESSION "daemon/BaseXsession=" EXPANDED_SYSCONFDIR "/gdm/Xsession"
+#define GDM_KEY_DEFAULTSESSION "daemon/DefaultSession=gnome.desktop"
 #define GDM_KEY_SUSPEND "daemon/SuspendCommand="
 
 #define GDM_KEY_UAUTHDIR "daemon/UserAuthDir="
@@ -151,6 +155,8 @@ enum {
 #define GDM_KEY_UAUTHFILE "daemon/UserAuthFile=.Xauthority"
 #define GDM_KEY_USER "daemon/User=gdm"
 
+#define GDM_KEY_DOUBLELOGINWARNING "daemon/DoubleLoginWarning=true"
+
 /* This defaults to true for backward compatibility,
  * it will not actually do timed login since the TimedLogin defaults
  * to nothing */
@@ -158,9 +164,9 @@ enum {
 #define GDM_KEY_TIMED_LOGIN "daemon/TimedLogin="
 #define GDM_KEY_TIMED_LOGIN_DELAY "daemon/TimedLoginDelay=30"
 
-#define GDM_KEY_STANDARD_XSERVER "daemon/StandardXServer=/usr/X11R6/bin/X"
+#define GDM_KEY_STANDARD_XSERVER "daemon/StandardXServer=" X_SERVER
 #define GDM_KEY_FLEXIBLE_XSERVERS "daemon/FlexibleXServers=5"
-#define GDM_KEY_XNEST "daemon/Xnest=/usr/X11R6/bin/Xnest -name Xnest"
+#define GDM_KEY_XNEST "daemon/Xnest=" X_SERVER_PATH "/Xnest -name Xnest"
 /* Keys for automatic VT allocation rather then letting it up to the
  * X server */
 #define GDM_KEY_FIRSTVT "daemon/FirstVT=7"
@@ -168,32 +174,34 @@ enum {
 
 /* Per server definitions */
 #define GDM_KEY_SERVER_NAME "name=Standard server"
-#define GDM_KEY_SERVER_COMMAND "command=/usr/X11R6/bin/X"
+#define GDM_KEY_SERVER_COMMAND "command=" X_SERVER
 /* runnable as flexi server */
 #define GDM_KEY_SERVER_FLEXIBLE "flexible=true"
 /* choosable from the login screen */
 #define GDM_KEY_SERVER_CHOOSABLE "choosable=false"
 /* Login is handled by gdm, otherwise it's a remote server */
 #define GDM_KEY_SERVER_HANDLED "handled=true"
+/* Instead of the greeter run the chooser */
+#define GDM_KEY_SERVER_CHOOSER "chooser=false"
 
 #define GDM_KEY_ALLOWROOT "security/AllowRoot=true"
 #define GDM_KEY_ALLOWREMOTEROOT "security/AllowRemoteRoot=true"
 #define GDM_KEY_ALLOWREMOTEAUTOLOGIN "security/AllowRemoteAutoLogin=false"
 #define GDM_KEY_MAXFILE "security/UserMaxFile=65536"
-#define GDM_KEY_SESSIONMAXFILE "security/SessionMaxFile=524288"
 #define GDM_KEY_RELAXPERM "security/RelaxPermissions=0"
 #define GDM_KEY_RETRYDELAY "security/RetryDelay=3"
+#define GDM_KEY_DISALLOWTCP "security/DisallowTCP=true"
 
 #define GDM_KEY_XDMCP "xdmcp/Enable=false"
 #define GDM_KEY_MAXPEND "xdmcp/MaxPending=4"
 #define GDM_KEY_MAXSESS "xdmcp/MaxSessions=16"
-#define GDM_KEY_MAXWAIT "xdmcp/MaxWait=30"
+#define GDM_KEY_MAXWAIT "xdmcp/MaxWait=15"
 #define GDM_KEY_DISPERHOST "xdmcp/DisplaysPerHost=2"
 #define GDM_KEY_UDPPORT "xdmcp/Port=177"
 #define GDM_KEY_INDIRECT "xdmcp/HonorIndirect=true"
 #define GDM_KEY_MAXINDIR "xdmcp/MaxPendingIndirect=4"
-#define GDM_KEY_MAXINDWAIT "xdmcp/MaxWaitIndirect=30"
-#define GDM_KEY_PINGINTERVAL "xdmcp/PingInterval=1"
+#define GDM_KEY_MAXINDWAIT "xdmcp/MaxWaitIndirect=15"
+#define GDM_KEY_PINGINTERVAL "xdmcp/PingIntervalSeconds=15"
 #define GDM_KEY_WILLING "xdmcp/Willing=" EXPANDED_SYSCONFDIR "/gdm/Xwilling"
 
 #define GDM_KEY_GTKRC "gui/GtkRC=" EXPANDED_DATADIR "/themes/Default/gtk/gtkrc"
@@ -201,19 +209,20 @@ enum {
 #define GDM_KEY_ICONHEIGHT "gui/MaxIconHeight=128"
 
 #define GDM_KEY_BROWSER "greeter/Browser=false"
-#define GDM_KEY_EXCLUDE "greeter/Exclude=bin,daemon,adm,lp,sync,shutdown,halt,mail,news,uucp,operator,nobody,gdm,postgres,pvm"
+#define GDM_KEY_EXCLUDE "greeter/Exclude=bin,daemon,adm,lp,sync,shutdown,halt,mail,news,uucp,operator,nobody,gdm,postgres,pvm,rpm,nfsnobody,pcap"
 #define GDM_KEY_MINIMALUID "greeter/MinimalUID=100"
-#define GDM_KEY_FACE "greeter/DefaultFace=" EXPANDED_PIXMAPDIR "nobody.png"
+#define GDM_KEY_FACE "greeter/DefaultFace=" EXPANDED_PIXMAPDIR "/nobody.png"
 #define GDM_KEY_FACEDIR "greeter/GlobalFaceDir=" EXPANDED_DATADIR "/faces/"
-#define GDM_KEY_ICON "greeter/Icon=" EXPANDED_PIXMAPDIR "/gdm.png"
 #define GDM_KEY_LOCFILE "greeter/LocaleFile=" EXPANDED_LOCALEDIR "/locale.alias"
 #define GDM_KEY_LOGO "greeter/Logo=" EXPANDED_PIXMAPDIR "/gdm-foot-logo.png"
 #define GDM_KEY_QUIVER "greeter/Quiver=true"
 #define GDM_KEY_SYSMENU "greeter/SystemMenu=true"
 #define GDM_KEY_CONFIGURATOR "daemon/Configurator=" EXPANDED_GDMCONFIGDIR "/gdmsetup --disable-sound --disable-crash-dialog"
 #define GDM_KEY_CONFIG_AVAILABLE "greeter/ConfigAvailable=true"
+#define GDM_KEY_CHOOSER_BUTTON "greeter/ChooserButton=true"
 #define GDM_KEY_TITLE_BAR "greeter/TitleBar=true"
-#define GDM_KEY_WELCOME "greeter/Welcome=Welcome to %n"
+#define GDM_KEY_WELCOME "greeter/Welcome=Welcome"
+#define GDM_KEY_REMOTEWELCOME "greeter/RemoteWelcome=Welcome to %n"
 #define GDM_KEY_XINERAMASCREEN "greeter/XineramaScreen=0"
 #define GDM_KEY_BACKGROUNDPROG "greeter/BackgroundProgram="
 #define GDM_KEY_RUNBACKGROUNDPROGALWAYS "greeter/RunBackgroundProgramAlways=false"
@@ -231,24 +240,28 @@ enum {
 #define GDM_KEY_GRAPHICAL_THEME "greeter/GraphicalTheme=circles"
 #define GDM_KEY_GRAPHICAL_THEME_DIR "greeter/GraphicalThemeDir=" EXPANDED_DATADIR "/gdm/themes/"
 
-#define GDM_KEY_SCAN "chooser/ScanTime=3"
+#define GDM_KEY_SCAN "chooser/ScanTime=4"
 #define GDM_KEY_HOST "chooser/DefaultHostImg=" EXPANDED_PIXMAPDIR "/nohost.png"
 #define GDM_KEY_HOSTDIR "chooser/HostImageDir=" EXPANDED_DATADIR "/hosts/"
 #define GDM_KEY_HOSTS "chooser/Hosts="
 #define GDM_KEY_BROADCAST "chooser/Broadcast=true"
+#define GDM_KEY_ALLOWADD "chooser/AllowAdd=true"
 
 #define GDM_KEY_DEBUG "debug/Enable=false"
 
 #define GDM_KEY_SERVERS "servers"
 
-#define GDM_KEY_SHOW_GNOME_CHOOSER "greeter/ShowGnomeChooserSession=true"
 #define GDM_KEY_SHOW_GNOME_FAILSAFE "greeter/ShowGnomeFailsafeSession=true"
 #define GDM_KEY_SHOW_XTERM_FAILSAFE "greeter/ShowXtermFailsafeSession=true"
 #define GDM_KEY_SHOW_LAST_SESSION "greeter/ShowLastSession=true"
 
 #define GDM_SESSION_FAILSAFE_GNOME "GDM_Failsafe.GNOME"
 #define GDM_SESSION_FAILSAFE_XTERM "GDM_Failsafe.XTERM"
-#define GDM_SESSION_GNOME_CHOOSER "Gnome Chooser"
+
+/* FIXME: will support these builtin types later */
+#define GDM_SESSION_DEFAULT "default"
+#define GDM_SESSION_CUSTOM "custom"
+#define GDM_SESSION_FAILSAFE "failsafe"
 
 #define GDM_STANDARD "Standard"
 
@@ -270,30 +283,42 @@ struct _GdmDisplay {
     gchar *authfile_gdm; /* authfile readable by gdm user
 			    if necessary */
     GSList *auths; 
+    GSList *local_auths; 
     gchar *userauth;
     gboolean authfb;
     gchar *command;
     gboolean failsafe_xserver;
-    gboolean use_chooser;
+    gboolean use_chooser; /* run chooser instead of greeter */
+    gchar *chosen_hostname; /* locally chosen hostname if not NULL,
+			       "-query chosen_hostname" is appened to server command line */
     guint indirect_id;
     gchar *cookie;
     gchar *bcookie;
     gchar *name;
     gchar *hostname;
     struct in_addr addr;
+    struct in_addr *addrs; /* array of addresses */
+    int addr_count; /* number of addresses in array */
+    /* Note that the above may in fact be empty even though
+       addr is set, these are just extra addresses
+       (it could also contain addr for all we know) */
+
     guint8 dispstat;
     guint16 dispnum;
     guint8 servstat;
     time_t starttime;
+    time_t managetime;
     guint8 type;
     pid_t greetpid;
     pid_t servpid;
     pid_t sesspid;
+    int last_sess_status; /* status returned by last session */
     pid_t slavepid;
     pid_t chooserpid;
     time_t acctime;
 
     gboolean handled;
+    gboolean tcp_disallowed;
 
     int vt;
 
@@ -302,6 +327,7 @@ struct _GdmDisplay {
     gboolean console;
 
     time_t last_start_time;
+    time_t last_loop_start_time;
     gint retry_count;
 
     int sleep_before_run;
@@ -309,6 +335,8 @@ struct _GdmDisplay {
     time_t last_x_failed;
     int x_faileds;
 
+    gboolean try_different_greeter;
+
     gboolean disabled;
 
     gboolean logged_in; /* TRUE if someone is logged in */
@@ -336,6 +364,15 @@ struct _GdmDisplay {
 
     /* order in the Xservers file for sessreg, -1 if unset yet */
     int x_servers_order;
+
+    /* The xsession-errors connection */
+    int xsession_errors_fd; /* write to the file */
+    int session_output_fd; /* read from the session */
+    int xsession_errors_bytes;
+#define MAX_XSESSION_ERRORS_BYTES (80*2500)  /* maximum number of bytes in
+						the ~/.xsession-errors file */
+    char *xsession_errors_filename; /* if NULL then there is no .xsession-errors
+				       file */
 };
 
 typedef struct _GdmXServer GdmXServer;
@@ -345,6 +382,7 @@ struct _GdmXServer {
 	char *command;
 	gboolean flexible;
 	gboolean choosable; /* not implemented yet */
+	gboolean chooser; /* instead of greeter, run chooser */
 	gboolean handled;
 };
 
@@ -405,9 +443,12 @@ void		gdm_final_cleanup	(void);
  * or gdmconfig or whatnot */
 
 /* The ones that pass a <slave pid> must be from a valid slave, and
- * the slave will be sent a SIGUSR2 */
+ * the slave will be sent a SIGUSR2.  Nowdays there is a pipe that is
+ * used from inside slaves, so those messages may stop being processed
+ * by the fifo at some point perhaps.  */
 /* The fifo protocol, used only by gdm internally */
 #define GDM_SOP_CHOSEN       "CHOSEN" /* <indirect id> <ip addr> */
+#define GDM_SOP_CHOSEN_LOCAL "CHOSEN_LOCAL" /* <slave pid> <hostname> */
 #define GDM_SOP_XPID         "XPID" /* <slave pid> <xpid> */
 #define GDM_SOP_SESSPID      "SESSPID" /* <slave pid> <sesspid> */
 #define GDM_SOP_GREETPID     "GREETPID" /* <slave pid> <greetpid> */
@@ -415,6 +456,9 @@ void		gdm_final_cleanup	(void);
 #define GDM_SOP_LOGGED_IN    "LOGGED_IN" /* <slave pid> <logged_in as int> */
 #define GDM_SOP_LOGIN        "LOGIN" /* <slave pid> <username> */
 #define GDM_SOP_COOKIE       "COOKIE" /* <slave pid> <cookie> */
+#define GDM_SOP_QUERYLOGIN   "QUERYLOGIN" /* <slave pid> <username> */
+/* if user already logged in somewhere, the ack response will be
+   <display>,<vt>,<display>,<vt>,... */
 #define GDM_SOP_DISP_NUM     "DISP_NUM" /* <slave pid> <display as int> */
 /* For linux only currently */
 #define GDM_SOP_VT_NUM       "VT_NUM" /* <slave pid> <vt as int> */
@@ -442,6 +486,8 @@ void		gdm_final_cleanup	(void);
  * by asking the user if they want to do that.  Note that this will not
  * kill any logged in sessions. */
 #define GDM_SOP_SOFT_RESTART_SERVERS "SOFT_RESTART_SERVERS"  /* no arguments */
+/* Suspend the machine if it is even allowed */
+#define GDM_SOP_SUSPEND_MACHINE "SUSPEND_MACHINE"  /* no arguments */
 
 /* Notification protocol */
 /* keys */
@@ -450,17 +496,20 @@ void		gdm_final_cleanup	(void);
 #define GDM_NOTIFY_ALLOWREMOTEAUTOLOGIN "AllowRemoteAutoLogin" /* <true/false as int> */
 #define GDM_NOTIFY_SYSMENU "SystemMenu" /* <true/false as int> */
 #define GDM_NOTIFY_CONFIG_AVAILABLE "ConfigAvailable" /* <true/false as int> */
+#define GDM_NOTIFY_CHOOSER_BUTTON "ChooserButton" /* <true/false as int> */
 #define GDM_NOTIFY_RETRYDELAY "RetryDelay" /* <seconds> */
 #define GDM_NOTIFY_GREETER "Greeter" /* <greeter binary> */
 #define GDM_NOTIFY_REMOTEGREETER "RemoteGreeter" /* <greeter binary> */
 #define GDM_NOTIFY_TIMED_LOGIN "TimedLogin" /* <login> */
 #define GDM_NOTIFY_TIMED_LOGIN_DELAY "TimedLoginDelay" /* <seconds> */
+#define GDM_NOTIFY_DISALLOWTCP "DisallowTCP" /* <true/false as int> */
 
 /* commands */
 #define GDM_NOTIFY_DIRTY_SERVERS "DIRTY_SERVERS"
 #define GDM_NOTIFY_SOFT_RESTART_SERVERS "SOFT_RESTART_SERVERS"
 
 /* Ack for a slave message */
+/* Note that an extra response can follow an 'ack' */
 #define GDM_SLAVE_NOTIFY_ACK 'A'
 /* Update this key */
 #define GDM_SLAVE_NOTIFY_KEY '!'
@@ -566,8 +615,22 @@ void		gdm_final_cleanup	(void);
  *   (that is, it is an xnest inside another console display) it is listed
  *   and instead of vt, it lists the parent display in standard form.
  */
+#define GDM_SUP_ALL_SERVERS  "ALL_SERVERS" /* None */
+/* ALL_SERVERS: List all servers, including console, remote, xnest.  This
+ * Can for example be useful to figure out if the server you are on is managed
+ * by the gdm daemon, by seeing if it is in the list.  It is also somewhat
+ * like the 'w' command but for graphical sessions.
+ * Supported since: 2.4.2.96
+ * Arguments:  None
+ * Answers:
+ *   OK <server>;<server>;...
+ *
+ *   <server> is <display>,<logged in user>
+ *
+ *   <logged in user> can be empty in case no one logged in yet
+ */
 #define GDM_SUP_UPDATE_CONFIG "UPDATE_CONFIG" /* <key> */
-/* AUTH_LOCAL: Tell the daemon to update config of some key.  Any user
+/* UPDATE_CONFIG: Tell the daemon to update config of some key.  Any user
  *             can really request that values are re-read but the daemon
  *             caches the last date of the config file so a user can't
  *             actually change any values unless they can write the
@@ -576,6 +639,7 @@ void		gdm_final_cleanup	(void);
  *   		 security/AllowRemoteRoot (2.3.90.2)
  *   		 security/AllowRemoteAutoLogin (2.3.90.2)
  *   		 security/RetryDelay (2.3.90.2)
+ *   		 security/DisallowTCP (2.4.2.0)
  *   		 daemon/Greeter (2.3.90.2)
  *   		 daemon/RemoteGreeter (2.3.90.2)
  *   		 xdmcp/Enable (2.3.90.2)
@@ -588,12 +652,13 @@ void		gdm_final_cleanup	(void);
  *			xdmcp/HonorIndirect
  *			xdmcp/MaxPendingIndirect
  *			xdmcp/MaxWaitIndirect
- *			xdmcp/PingInterval (only affects new connections)
+ *			xdmcp/PingIntervalSeconds (only affects new connections)
  *   		 xdmcp/TimedLogin (2.3.90.3)
  *   		 xdmcp/TimedLoginEnable (2.3.90.3)
  *   		 xdmcp/TimedLoginDelay (2.3.90.3)
  *   		 greeter/SystemMenu (2.3.90.3)
  *   		 greeter/ConfigAvailable (2.3.90.3)
+ *   		 greeter/ChooserButton (2.4.2.0)
  * Supported since: 2.3.90.2
  * Arguments:  <key>
  *   <key> is just the base part of the key such as "security/AllowRemoteRoot"
@@ -627,7 +692,8 @@ enum {
 	do {		\
 		errno = 0;	\
 		expr;		\
-	} while (errno == EINTR);
+	} while G_UNLIKELY (errno == EINTR);
+		
 
 #endif /* GDM_H */
 
